@@ -32,13 +32,14 @@ r"""
     The graph is inserted as a PNG+image map into HTML and a PDF in
     LaTeX.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 import sys
 import inspect
+import __builtin__
 try:
     from hashlib import md5
 except ImportError:
@@ -47,7 +48,8 @@ except ImportError:
 from docutils import nodes
 from docutils.parsers.rst import directives
 
-from sphinx.ext.graphviz import render_dot_html, render_dot_latex
+from sphinx.ext.graphviz import render_dot_html, render_dot_latex, \
+    render_dot_texinfo
 from sphinx.util.compat import Directive
 
 
@@ -66,27 +68,26 @@ class InheritanceGraph(object):
     from all the way to the root "object", and then is able to generate a
     graphviz dot graph from them.
     """
-    def __init__(self, class_names, currmodule, show_builtins=False, parts=0):
-        """
-        *class_names* is a list of child classes to show bases from.
+    def __init__(self, class_names, currmodule, show_builtins=False,
+                 private_bases=False, parts=0):
+        """*class_names* is a list of child classes to show bases from.
 
         If *show_builtins* is True, then Python builtins will be shown
         in the graph.
         """
         self.class_names = class_names
         classes = self._import_classes(class_names, currmodule)
-        self.class_info = self._class_info(classes, show_builtins, parts)
+        self.class_info = self._class_info(classes, show_builtins,
+                                           private_bases, parts)
         if not self.class_info:
             raise InheritanceException('No classes found for '
                                        'inheritance diagram')
 
     def _import_class_or_module(self, name, currmodule):
-        """
-        Import a class using its fully-qualified *name*.
-        """
+        """Import a class using its fully-qualified *name*."""
         try:
             path, base = class_sig_re.match(name).groups()
-        except ValueError:
+        except (AttributeError, ValueError):
             raise InheritanceException('Invalid class or module %r specified '
                                        'for inheritance diagram' % name)
 
@@ -134,7 +135,7 @@ class InheritanceGraph(object):
             classes.extend(self._import_class_or_module(name, currmodule))
         return classes
 
-    def _class_info(self, classes, show_builtins, parts):
+    def _class_info(self, classes, show_builtins, private_bases, parts):
         """Return name and bases for all classes that are ancestors of
         *classes*.
 
@@ -142,11 +143,13 @@ class InheritanceGraph(object):
         displayed node names.
         """
         all_classes = {}
-        builtins = __builtins__.values()
+        builtins = vars(__builtin__).values()
 
         def recurse(cls):
             if not show_builtins and cls in builtins:
                 return
+            if not private_bases and cls.__name__.startswith('_'):
+                return
 
             nodename = self.class_name(cls, parts)
             fullname = self.class_name(cls, 0)
@@ -155,7 +158,9 @@ class InheritanceGraph(object):
             all_classes[cls] = (nodename, fullname, baselist)
             for base in cls.__bases__:
                 if not show_builtins and base in builtins:
-                    return
+                    continue
+                if not private_bases and base.__name__.startswith('_'):
+                    continue
                 baselist.append(self.class_name(base, parts))
                 if base not in all_classes:
                     recurse(base)
@@ -182,9 +187,7 @@ class InheritanceGraph(object):
         return '.'.join(name_parts[-parts:])
 
     def get_all_class_names(self):
-        """
-        Get all of the class names involved in the graph.
-        """
+        """Get all of the class names involved in the graph."""
         return [fullname for (_, fullname, _) in self.class_info]
 
     # These are the default attrs for graphviz
@@ -213,9 +216,8 @@ class InheritanceGraph(object):
 
     def generate_dot(self, name, urls={}, env=None,
                      graph_attrs={}, node_attrs={}, edge_attrs={}):
-        """
-        Generate a graphviz dot graph from the classes that
-        were passed in to __init__.
+        """Generate a graphviz dot graph from the classes that were passed in
+        to __init__.
 
         *name* is the name of the graph.
 
@@ -274,6 +276,7 @@ class InheritanceDiagram(Directive):
     final_argument_whitespace = True
     option_spec = {
         'parts': directives.nonnegative_int,
+        'private-bases': directives.flag,
     }
 
     def run(self):
@@ -290,7 +293,8 @@ class InheritanceDiagram(Directive):
         try:
             graph = InheritanceGraph(
                 class_names, env.temp_data.get('py:module'),
-                parts=node['parts'])
+                parts=node['parts'],
+                private_bases='private-bases' in self.options)
         except InheritanceException, err:
             return [node.document.reporter.warning(err.args[0],
                                                    line=self.lineno)]
@@ -352,6 +356,21 @@ def latex_visit_inheritance_diagram(self, node):
     raise nodes.SkipNode
 
 
+def texinfo_visit_inheritance_diagram(self, node):
+    """
+    Output the graph for Texinfo.  This will insert a PNG.
+    """
+    graph = node['graph']
+
+    graph_hash = get_graph_hash(node)
+    name = 'inheritance%s' % graph_hash
+
+    dotcode = graph.generate_dot(name, env=self.builder.env,
+                                 graph_attrs={'size': '"6.0,6.0"'})
+    render_dot_texinfo(self, node, dotcode, [], 'inheritance')
+    raise nodes.SkipNode
+
+
 def skip(self, node):
     raise nodes.SkipNode
 
@@ -363,7 +382,8 @@ def setup(app):
         latex=(latex_visit_inheritance_diagram, None),
         html=(html_visit_inheritance_diagram, None),
         text=(skip, None),
-        man=(skip, None))
+        man=(skip, None),
+        texinfo=(texinfo_visit_inheritance_diagram, None))
     app.add_directive('inheritance-diagram', InheritanceDiagram)
     app.add_config_value('inheritance_graph_attrs', {}, False),
     app.add_config_value('inheritance_node_attrs', {}, False),
